# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['FirewallOptionsArgs', 'FirewallOptions']

@pulumi.input_type
class FirewallOptionsArgs:
    def __init__(__self__, *,
                 node_name: pulumi.Input[str],
                 container_id: Optional[pulumi.Input[int]] = None,
                 dhcp: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 input_policy: Optional[pulumi.Input[str]] = None,
                 ipfilter: Optional[pulumi.Input[bool]] = None,
                 log_level_in: Optional[pulumi.Input[str]] = None,
                 log_level_out: Optional[pulumi.Input[str]] = None,
                 macfilter: Optional[pulumi.Input[bool]] = None,
                 ndp: Optional[pulumi.Input[bool]] = None,
                 output_policy: Optional[pulumi.Input[str]] = None,
                 radv: Optional[pulumi.Input[bool]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a FirewallOptions resource.
        :param pulumi.Input[str] node_name: Node name.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[bool] dhcp: Enable DHCP.
        :param pulumi.Input[bool] enabled: Enable or disable the firewall.
        :param pulumi.Input[str] input_policy: The default input
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] ipfilter: Enable default IP filters. This is equivalent to
               adding an empty `ipfilter-net<id>` ipset for every interface. Such ipsets
               implicitly contain sane default restrictions such as restricting IPv6 link
               local addresses to the one derived from the interface's MAC address. For
               containers the configured IP addresses will be implicitly added.
        :param pulumi.Input[str] log_level_in: Log level for incoming
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[str] log_level_out: Log level for outgoing
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[bool] macfilter: Enable/disable MAC address filter.
        :param pulumi.Input[bool] ndp: Enable NDP (Neighbor Discovery Protocol).
        :param pulumi.Input[str] output_policy: The default output
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] radv: Enable Router Advertisement.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        pulumi.set(__self__, "node_name", node_name)
        if container_id is not None:
            pulumi.set(__self__, "container_id", container_id)
        if dhcp is not None:
            pulumi.set(__self__, "dhcp", dhcp)
        if enabled is not None:
            pulumi.set(__self__, "enabled", enabled)
        if input_policy is not None:
            pulumi.set(__self__, "input_policy", input_policy)
        if ipfilter is not None:
            pulumi.set(__self__, "ipfilter", ipfilter)
        if log_level_in is not None:
            pulumi.set(__self__, "log_level_in", log_level_in)
        if log_level_out is not None:
            pulumi.set(__self__, "log_level_out", log_level_out)
        if macfilter is not None:
            pulumi.set(__self__, "macfilter", macfilter)
        if ndp is not None:
            pulumi.set(__self__, "ndp", ndp)
        if output_policy is not None:
            pulumi.set(__self__, "output_policy", output_policy)
        if radv is not None:
            pulumi.set(__self__, "radv", radv)
        if vm_id is not None:
            pulumi.set(__self__, "vm_id", vm_id)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Input[str]:
        """
        Node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> Optional[pulumi.Input[int]]:
        """
        Container ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "container_id")

    @container_id.setter
    def container_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "container_id", value)

    @property
    @pulumi.getter
    def dhcp(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable DHCP.
        """
        return pulumi.get(self, "dhcp")

    @dhcp.setter
    def dhcp(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dhcp", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable or disable the firewall.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter(name="inputPolicy")
    def input_policy(self) -> Optional[pulumi.Input[str]]:
        """
        The default input
        policy (`ACCEPT`, `DROP`, `REJECT`).
        """
        return pulumi.get(self, "input_policy")

    @input_policy.setter
    def input_policy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "input_policy", value)

    @property
    @pulumi.getter
    def ipfilter(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable default IP filters. This is equivalent to
        adding an empty `ipfilter-net<id>` ipset for every interface. Such ipsets
        implicitly contain sane default restrictions such as restricting IPv6 link
        local addresses to the one derived from the interface's MAC address. For
        containers the configured IP addresses will be implicitly added.
        """
        return pulumi.get(self, "ipfilter")

    @ipfilter.setter
    def ipfilter(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ipfilter", value)

    @property
    @pulumi.getter(name="logLevelIn")
    def log_level_in(self) -> Optional[pulumi.Input[str]]:
        """
        Log level for incoming
        packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
        `debug`, `nolog`).
        """
        return pulumi.get(self, "log_level_in")

    @log_level_in.setter
    def log_level_in(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "log_level_in", value)

    @property
    @pulumi.getter(name="logLevelOut")
    def log_level_out(self) -> Optional[pulumi.Input[str]]:
        """
        Log level for outgoing
        packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
        `debug`, `nolog`).
        """
        return pulumi.get(self, "log_level_out")

    @log_level_out.setter
    def log_level_out(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "log_level_out", value)

    @property
    @pulumi.getter
    def macfilter(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable/disable MAC address filter.
        """
        return pulumi.get(self, "macfilter")

    @macfilter.setter
    def macfilter(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "macfilter", value)

    @property
    @pulumi.getter
    def ndp(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable NDP (Neighbor Discovery Protocol).
        """
        return pulumi.get(self, "ndp")

    @ndp.setter
    def ndp(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ndp", value)

    @property
    @pulumi.getter(name="outputPolicy")
    def output_policy(self) -> Optional[pulumi.Input[str]]:
        """
        The default output
        policy (`ACCEPT`, `DROP`, `REJECT`).
        """
        return pulumi.get(self, "output_policy")

    @output_policy.setter
    def output_policy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "output_policy", value)

    @property
    @pulumi.getter
    def radv(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable Router Advertisement.
        """
        return pulumi.get(self, "radv")

    @radv.setter
    def radv(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "radv", value)

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[pulumi.Input[int]]:
        """
        VM ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "vm_id")

    @vm_id.setter
    def vm_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vm_id", value)


@pulumi.input_type
class _FirewallOptionsState:
    def __init__(__self__, *,
                 container_id: Optional[pulumi.Input[int]] = None,
                 dhcp: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 input_policy: Optional[pulumi.Input[str]] = None,
                 ipfilter: Optional[pulumi.Input[bool]] = None,
                 log_level_in: Optional[pulumi.Input[str]] = None,
                 log_level_out: Optional[pulumi.Input[str]] = None,
                 macfilter: Optional[pulumi.Input[bool]] = None,
                 ndp: Optional[pulumi.Input[bool]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 output_policy: Optional[pulumi.Input[str]] = None,
                 radv: Optional[pulumi.Input[bool]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering FirewallOptions resources.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[bool] dhcp: Enable DHCP.
        :param pulumi.Input[bool] enabled: Enable or disable the firewall.
        :param pulumi.Input[str] input_policy: The default input
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] ipfilter: Enable default IP filters. This is equivalent to
               adding an empty `ipfilter-net<id>` ipset for every interface. Such ipsets
               implicitly contain sane default restrictions such as restricting IPv6 link
               local addresses to the one derived from the interface's MAC address. For
               containers the configured IP addresses will be implicitly added.
        :param pulumi.Input[str] log_level_in: Log level for incoming
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[str] log_level_out: Log level for outgoing
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[bool] macfilter: Enable/disable MAC address filter.
        :param pulumi.Input[bool] ndp: Enable NDP (Neighbor Discovery Protocol).
        :param pulumi.Input[str] node_name: Node name.
        :param pulumi.Input[str] output_policy: The default output
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] radv: Enable Router Advertisement.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        if container_id is not None:
            pulumi.set(__self__, "container_id", container_id)
        if dhcp is not None:
            pulumi.set(__self__, "dhcp", dhcp)
        if enabled is not None:
            pulumi.set(__self__, "enabled", enabled)
        if input_policy is not None:
            pulumi.set(__self__, "input_policy", input_policy)
        if ipfilter is not None:
            pulumi.set(__self__, "ipfilter", ipfilter)
        if log_level_in is not None:
            pulumi.set(__self__, "log_level_in", log_level_in)
        if log_level_out is not None:
            pulumi.set(__self__, "log_level_out", log_level_out)
        if macfilter is not None:
            pulumi.set(__self__, "macfilter", macfilter)
        if ndp is not None:
            pulumi.set(__self__, "ndp", ndp)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if output_policy is not None:
            pulumi.set(__self__, "output_policy", output_policy)
        if radv is not None:
            pulumi.set(__self__, "radv", radv)
        if vm_id is not None:
            pulumi.set(__self__, "vm_id", vm_id)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> Optional[pulumi.Input[int]]:
        """
        Container ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "container_id")

    @container_id.setter
    def container_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "container_id", value)

    @property
    @pulumi.getter
    def dhcp(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable DHCP.
        """
        return pulumi.get(self, "dhcp")

    @dhcp.setter
    def dhcp(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dhcp", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable or disable the firewall.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter(name="inputPolicy")
    def input_policy(self) -> Optional[pulumi.Input[str]]:
        """
        The default input
        policy (`ACCEPT`, `DROP`, `REJECT`).
        """
        return pulumi.get(self, "input_policy")

    @input_policy.setter
    def input_policy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "input_policy", value)

    @property
    @pulumi.getter
    def ipfilter(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable default IP filters. This is equivalent to
        adding an empty `ipfilter-net<id>` ipset for every interface. Such ipsets
        implicitly contain sane default restrictions such as restricting IPv6 link
        local addresses to the one derived from the interface's MAC address. For
        containers the configured IP addresses will be implicitly added.
        """
        return pulumi.get(self, "ipfilter")

    @ipfilter.setter
    def ipfilter(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ipfilter", value)

    @property
    @pulumi.getter(name="logLevelIn")
    def log_level_in(self) -> Optional[pulumi.Input[str]]:
        """
        Log level for incoming
        packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
        `debug`, `nolog`).
        """
        return pulumi.get(self, "log_level_in")

    @log_level_in.setter
    def log_level_in(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "log_level_in", value)

    @property
    @pulumi.getter(name="logLevelOut")
    def log_level_out(self) -> Optional[pulumi.Input[str]]:
        """
        Log level for outgoing
        packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
        `debug`, `nolog`).
        """
        return pulumi.get(self, "log_level_out")

    @log_level_out.setter
    def log_level_out(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "log_level_out", value)

    @property
    @pulumi.getter
    def macfilter(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable/disable MAC address filter.
        """
        return pulumi.get(self, "macfilter")

    @macfilter.setter
    def macfilter(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "macfilter", value)

    @property
    @pulumi.getter
    def ndp(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable NDP (Neighbor Discovery Protocol).
        """
        return pulumi.get(self, "ndp")

    @ndp.setter
    def ndp(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ndp", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        Node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter(name="outputPolicy")
    def output_policy(self) -> Optional[pulumi.Input[str]]:
        """
        The default output
        policy (`ACCEPT`, `DROP`, `REJECT`).
        """
        return pulumi.get(self, "output_policy")

    @output_policy.setter
    def output_policy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "output_policy", value)

    @property
    @pulumi.getter
    def radv(self) -> Optional[pulumi.Input[bool]]:
        """
        Enable Router Advertisement.
        """
        return pulumi.get(self, "radv")

    @radv.setter
    def radv(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "radv", value)

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[pulumi.Input[int]]:
        """
        VM ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "vm_id")

    @vm_id.setter
    def vm_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vm_id", value)


class FirewallOptions(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 dhcp: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 input_policy: Optional[pulumi.Input[str]] = None,
                 ipfilter: Optional[pulumi.Input[bool]] = None,
                 log_level_in: Optional[pulumi.Input[str]] = None,
                 log_level_out: Optional[pulumi.Input[str]] = None,
                 macfilter: Optional[pulumi.Input[bool]] = None,
                 ndp: Optional[pulumi.Input[bool]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 output_policy: Optional[pulumi.Input[str]] = None,
                 radv: Optional[pulumi.Input[bool]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        Manages firewall options on VM / Container level.

        ## Example Usage

        <!--Start PulumiCodeChooser -->
        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        example = proxmoxve.network.FirewallOptions("example",
            node_name=proxmox_virtual_environment_vm["example"]["node_name"],
            vm_id=proxmox_virtual_environment_vm["example"]["vm_id"],
            dhcp=True,
            enabled=False,
            ipfilter=True,
            log_level_in="info",
            log_level_out="info",
            macfilter=False,
            ndp=True,
            input_policy="ACCEPT",
            output_policy="ACCEPT",
            radv=True,
            opts=pulumi.ResourceOptions(depends_on=[proxmox_virtual_environment_vm["example"]]))
        ```
        <!--End PulumiCodeChooser -->

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[bool] dhcp: Enable DHCP.
        :param pulumi.Input[bool] enabled: Enable or disable the firewall.
        :param pulumi.Input[str] input_policy: The default input
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] ipfilter: Enable default IP filters. This is equivalent to
               adding an empty `ipfilter-net<id>` ipset for every interface. Such ipsets
               implicitly contain sane default restrictions such as restricting IPv6 link
               local addresses to the one derived from the interface's MAC address. For
               containers the configured IP addresses will be implicitly added.
        :param pulumi.Input[str] log_level_in: Log level for incoming
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[str] log_level_out: Log level for outgoing
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[bool] macfilter: Enable/disable MAC address filter.
        :param pulumi.Input[bool] ndp: Enable NDP (Neighbor Discovery Protocol).
        :param pulumi.Input[str] node_name: Node name.
        :param pulumi.Input[str] output_policy: The default output
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] radv: Enable Router Advertisement.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: FirewallOptionsArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages firewall options on VM / Container level.

        ## Example Usage

        <!--Start PulumiCodeChooser -->
        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        example = proxmoxve.network.FirewallOptions("example",
            node_name=proxmox_virtual_environment_vm["example"]["node_name"],
            vm_id=proxmox_virtual_environment_vm["example"]["vm_id"],
            dhcp=True,
            enabled=False,
            ipfilter=True,
            log_level_in="info",
            log_level_out="info",
            macfilter=False,
            ndp=True,
            input_policy="ACCEPT",
            output_policy="ACCEPT",
            radv=True,
            opts=pulumi.ResourceOptions(depends_on=[proxmox_virtual_environment_vm["example"]]))
        ```
        <!--End PulumiCodeChooser -->

        :param str resource_name: The name of the resource.
        :param FirewallOptionsArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FirewallOptionsArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 dhcp: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 input_policy: Optional[pulumi.Input[str]] = None,
                 ipfilter: Optional[pulumi.Input[bool]] = None,
                 log_level_in: Optional[pulumi.Input[str]] = None,
                 log_level_out: Optional[pulumi.Input[str]] = None,
                 macfilter: Optional[pulumi.Input[bool]] = None,
                 ndp: Optional[pulumi.Input[bool]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 output_policy: Optional[pulumi.Input[str]] = None,
                 radv: Optional[pulumi.Input[bool]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FirewallOptionsArgs.__new__(FirewallOptionsArgs)

            __props__.__dict__["container_id"] = container_id
            __props__.__dict__["dhcp"] = dhcp
            __props__.__dict__["enabled"] = enabled
            __props__.__dict__["input_policy"] = input_policy
            __props__.__dict__["ipfilter"] = ipfilter
            __props__.__dict__["log_level_in"] = log_level_in
            __props__.__dict__["log_level_out"] = log_level_out
            __props__.__dict__["macfilter"] = macfilter
            __props__.__dict__["ndp"] = ndp
            if node_name is None and not opts.urn:
                raise TypeError("Missing required property 'node_name'")
            __props__.__dict__["node_name"] = node_name
            __props__.__dict__["output_policy"] = output_policy
            __props__.__dict__["radv"] = radv
            __props__.__dict__["vm_id"] = vm_id
        super(FirewallOptions, __self__).__init__(
            'proxmoxve:Network/firewallOptions:FirewallOptions',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            container_id: Optional[pulumi.Input[int]] = None,
            dhcp: Optional[pulumi.Input[bool]] = None,
            enabled: Optional[pulumi.Input[bool]] = None,
            input_policy: Optional[pulumi.Input[str]] = None,
            ipfilter: Optional[pulumi.Input[bool]] = None,
            log_level_in: Optional[pulumi.Input[str]] = None,
            log_level_out: Optional[pulumi.Input[str]] = None,
            macfilter: Optional[pulumi.Input[bool]] = None,
            ndp: Optional[pulumi.Input[bool]] = None,
            node_name: Optional[pulumi.Input[str]] = None,
            output_policy: Optional[pulumi.Input[str]] = None,
            radv: Optional[pulumi.Input[bool]] = None,
            vm_id: Optional[pulumi.Input[int]] = None) -> 'FirewallOptions':
        """
        Get an existing FirewallOptions resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[bool] dhcp: Enable DHCP.
        :param pulumi.Input[bool] enabled: Enable or disable the firewall.
        :param pulumi.Input[str] input_policy: The default input
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] ipfilter: Enable default IP filters. This is equivalent to
               adding an empty `ipfilter-net<id>` ipset for every interface. Such ipsets
               implicitly contain sane default restrictions such as restricting IPv6 link
               local addresses to the one derived from the interface's MAC address. For
               containers the configured IP addresses will be implicitly added.
        :param pulumi.Input[str] log_level_in: Log level for incoming
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[str] log_level_out: Log level for outgoing
               packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
               `debug`, `nolog`).
        :param pulumi.Input[bool] macfilter: Enable/disable MAC address filter.
        :param pulumi.Input[bool] ndp: Enable NDP (Neighbor Discovery Protocol).
        :param pulumi.Input[str] node_name: Node name.
        :param pulumi.Input[str] output_policy: The default output
               policy (`ACCEPT`, `DROP`, `REJECT`).
        :param pulumi.Input[bool] radv: Enable Router Advertisement.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FirewallOptionsState.__new__(_FirewallOptionsState)

        __props__.__dict__["container_id"] = container_id
        __props__.__dict__["dhcp"] = dhcp
        __props__.__dict__["enabled"] = enabled
        __props__.__dict__["input_policy"] = input_policy
        __props__.__dict__["ipfilter"] = ipfilter
        __props__.__dict__["log_level_in"] = log_level_in
        __props__.__dict__["log_level_out"] = log_level_out
        __props__.__dict__["macfilter"] = macfilter
        __props__.__dict__["ndp"] = ndp
        __props__.__dict__["node_name"] = node_name
        __props__.__dict__["output_policy"] = output_policy
        __props__.__dict__["radv"] = radv
        __props__.__dict__["vm_id"] = vm_id
        return FirewallOptions(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> pulumi.Output[Optional[int]]:
        """
        Container ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "container_id")

    @property
    @pulumi.getter
    def dhcp(self) -> pulumi.Output[Optional[bool]]:
        """
        Enable DHCP.
        """
        return pulumi.get(self, "dhcp")

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Output[Optional[bool]]:
        """
        Enable or disable the firewall.
        """
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter(name="inputPolicy")
    def input_policy(self) -> pulumi.Output[Optional[str]]:
        """
        The default input
        policy (`ACCEPT`, `DROP`, `REJECT`).
        """
        return pulumi.get(self, "input_policy")

    @property
    @pulumi.getter
    def ipfilter(self) -> pulumi.Output[Optional[bool]]:
        """
        Enable default IP filters. This is equivalent to
        adding an empty `ipfilter-net<id>` ipset for every interface. Such ipsets
        implicitly contain sane default restrictions such as restricting IPv6 link
        local addresses to the one derived from the interface's MAC address. For
        containers the configured IP addresses will be implicitly added.
        """
        return pulumi.get(self, "ipfilter")

    @property
    @pulumi.getter(name="logLevelIn")
    def log_level_in(self) -> pulumi.Output[Optional[str]]:
        """
        Log level for incoming
        packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
        `debug`, `nolog`).
        """
        return pulumi.get(self, "log_level_in")

    @property
    @pulumi.getter(name="logLevelOut")
    def log_level_out(self) -> pulumi.Output[Optional[str]]:
        """
        Log level for outgoing
        packets (`emerg`, `alert`, `crit`, `err`, `warning`, `notice`, `info`,
        `debug`, `nolog`).
        """
        return pulumi.get(self, "log_level_out")

    @property
    @pulumi.getter
    def macfilter(self) -> pulumi.Output[Optional[bool]]:
        """
        Enable/disable MAC address filter.
        """
        return pulumi.get(self, "macfilter")

    @property
    @pulumi.getter
    def ndp(self) -> pulumi.Output[Optional[bool]]:
        """
        Enable NDP (Neighbor Discovery Protocol).
        """
        return pulumi.get(self, "ndp")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Output[str]:
        """
        Node name.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter(name="outputPolicy")
    def output_policy(self) -> pulumi.Output[Optional[str]]:
        """
        The default output
        policy (`ACCEPT`, `DROP`, `REJECT`).
        """
        return pulumi.get(self, "output_policy")

    @property
    @pulumi.getter
    def radv(self) -> pulumi.Output[Optional[bool]]:
        """
        Enable Router Advertisement.
        """
        return pulumi.get(self, "radv")

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> pulumi.Output[Optional[int]]:
        """
        VM ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "vm_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['FirewallRulesArgs', 'FirewallRules']

@pulumi.input_type
class FirewallRulesArgs:
    def __init__(__self__, *,
                 rules: pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]],
                 container_id: Optional[pulumi.Input[int]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a FirewallRules resource.
        :param pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]] rules: Firewall rule block (multiple blocks supported).
               The provider supports two types of the `rule` blocks:
               - A rule definition block, which includes the following arguments:
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level
               rules.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level rules.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level rules.
        """
        pulumi.set(__self__, "rules", rules)
        if container_id is not None:
            pulumi.set(__self__, "container_id", container_id)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if vm_id is not None:
            pulumi.set(__self__, "vm_id", vm_id)

    @property
    @pulumi.getter
    def rules(self) -> pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]]:
        """
        Firewall rule block (multiple blocks supported).
        The provider supports two types of the `rule` blocks:
        - A rule definition block, which includes the following arguments:
        """
        return pulumi.get(self, "rules")

    @rules.setter
    def rules(self, value: pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]]):
        pulumi.set(self, "rules", value)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> Optional[pulumi.Input[int]]:
        """
        Container ID. Leave empty for cluster level
        rules.
        """
        return pulumi.get(self, "container_id")

    @container_id.setter
    def container_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "container_id", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        Node name. Leave empty for cluster level rules.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[pulumi.Input[int]]:
        """
        VM ID. Leave empty for cluster level rules.
        """
        return pulumi.get(self, "vm_id")

    @vm_id.setter
    def vm_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vm_id", value)


@pulumi.input_type
class _FirewallRulesState:
    def __init__(__self__, *,
                 container_id: Optional[pulumi.Input[int]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering FirewallRules resources.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level
               rules.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level rules.
        :param pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]] rules: Firewall rule block (multiple blocks supported).
               The provider supports two types of the `rule` blocks:
               - A rule definition block, which includes the following arguments:
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level rules.
        """
        if container_id is not None:
            pulumi.set(__self__, "container_id", container_id)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if rules is not None:
            pulumi.set(__self__, "rules", rules)
        if vm_id is not None:
            pulumi.set(__self__, "vm_id", vm_id)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> Optional[pulumi.Input[int]]:
        """
        Container ID. Leave empty for cluster level
        rules.
        """
        return pulumi.get(self, "container_id")

    @container_id.setter
    def container_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "container_id", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        Node name. Leave empty for cluster level rules.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter
    def rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]]]:
        """
        Firewall rule block (multiple blocks supported).
        The provider supports two types of the `rule` blocks:
        - A rule definition block, which includes the following arguments:
        """
        return pulumi.get(self, "rules")

    @rules.setter
    def rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallRulesRuleArgs']]]]):
        pulumi.set(self, "rules", value)

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[pulumi.Input[int]]:
        """
        VM ID. Leave empty for cluster level rules.
        """
        return pulumi.get(self, "vm_id")

    @vm_id.setter
    def vm_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vm_id", value)


class FirewallRules(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallRulesRuleArgs']]]]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        A security group is a collection of rules, defined at cluster level, which can
        be used in all VMs' rules. For example, you can define a group named “webserver”
        with rules to open the http and https ports. Rules can be created on the cluster
        level, on VM / Container level.

        ## Example Usage

        <!--Start PulumiCodeChooser -->
        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        inbound = proxmoxve.network.FirewallRules("inbound",
            node_name=proxmox_virtual_environment_vm["example"]["node_name"],
            vm_id=proxmox_virtual_environment_vm["example"]["vm_id"],
            rules=[
                proxmoxve.network.FirewallRulesRuleArgs(
                    type="in",
                    action="ACCEPT",
                    comment="Allow HTTP",
                    dest="192.168.1.5",
                    dport="80",
                    proto="tcp",
                    log="info",
                ),
                proxmoxve.network.FirewallRulesRuleArgs(
                    type="in",
                    action="ACCEPT",
                    comment="Allow HTTPS",
                    dest="192.168.1.5",
                    dport="443",
                    proto="tcp",
                    log="info",
                ),
                proxmoxve.network.FirewallRulesRuleArgs(
                    security_group=proxmox_virtual_environment_cluster_firewall_security_group["example"]["name"],
                    comment="From security group",
                    iface="net0",
                ),
            ],
            opts=pulumi.ResourceOptions(depends_on=[
                    proxmox_virtual_environment_vm["example"],
                    proxmox_virtual_environment_cluster_firewall_security_group["example"],
                ]))
        ```
        <!--End PulumiCodeChooser -->

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level
               rules.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level rules.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallRulesRuleArgs']]]] rules: Firewall rule block (multiple blocks supported).
               The provider supports two types of the `rule` blocks:
               - A rule definition block, which includes the following arguments:
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level rules.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: FirewallRulesArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        A security group is a collection of rules, defined at cluster level, which can
        be used in all VMs' rules. For example, you can define a group named “webserver”
        with rules to open the http and https ports. Rules can be created on the cluster
        level, on VM / Container level.

        ## Example Usage

        <!--Start PulumiCodeChooser -->
        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        inbound = proxmoxve.network.FirewallRules("inbound",
            node_name=proxmox_virtual_environment_vm["example"]["node_name"],
            vm_id=proxmox_virtual_environment_vm["example"]["vm_id"],
            rules=[
                proxmoxve.network.FirewallRulesRuleArgs(
                    type="in",
                    action="ACCEPT",
                    comment="Allow HTTP",
                    dest="192.168.1.5",
                    dport="80",
                    proto="tcp",
                    log="info",
                ),
                proxmoxve.network.FirewallRulesRuleArgs(
                    type="in",
                    action="ACCEPT",
                    comment="Allow HTTPS",
                    dest="192.168.1.5",
                    dport="443",
                    proto="tcp",
                    log="info",
                ),
                proxmoxve.network.FirewallRulesRuleArgs(
                    security_group=proxmox_virtual_environment_cluster_firewall_security_group["example"]["name"],
                    comment="From security group",
                    iface="net0",
                ),
            ],
            opts=pulumi.ResourceOptions(depends_on=[
                    proxmox_virtual_environment_vm["example"],
                    proxmox_virtual_environment_cluster_firewall_security_group["example"],
                ]))
        ```
        <!--End PulumiCodeChooser -->

        :param str resource_name: The name of the resource.
        :param FirewallRulesArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FirewallRulesArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallRulesRuleArgs']]]]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FirewallRulesArgs.__new__(FirewallRulesArgs)

            __props__.__dict__["container_id"] = container_id
            __props__.__dict__["node_name"] = node_name
            if rules is None and not opts.urn:
                raise TypeError("Missing required property 'rules'")
            __props__.__dict__["rules"] = rules
            __props__.__dict__["vm_id"] = vm_id
        super(FirewallRules, __self__).__init__(
            'proxmoxve:Network/firewallRules:FirewallRules',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            container_id: Optional[pulumi.Input[int]] = None,
            node_name: Optional[pulumi.Input[str]] = None,
            rules: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallRulesRuleArgs']]]]] = None,
            vm_id: Optional[pulumi.Input[int]] = None) -> 'FirewallRules':
        """
        Get an existing FirewallRules resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level
               rules.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level rules.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallRulesRuleArgs']]]] rules: Firewall rule block (multiple blocks supported).
               The provider supports two types of the `rule` blocks:
               - A rule definition block, which includes the following arguments:
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level rules.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FirewallRulesState.__new__(_FirewallRulesState)

        __props__.__dict__["container_id"] = container_id
        __props__.__dict__["node_name"] = node_name
        __props__.__dict__["rules"] = rules
        __props__.__dict__["vm_id"] = vm_id
        return FirewallRules(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> pulumi.Output[Optional[int]]:
        """
        Container ID. Leave empty for cluster level
        rules.
        """
        return pulumi.get(self, "container_id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Output[Optional[str]]:
        """
        Node name. Leave empty for cluster level rules.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def rules(self) -> pulumi.Output[Sequence['outputs.FirewallRulesRule']]:
        """
        Firewall rule block (multiple blocks supported).
        The provider supports two types of the `rule` blocks:
        - A rule definition block, which includes the following arguments:
        """
        return pulumi.get(self, "rules")

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> pulumi.Output[Optional[int]]:
        """
        VM ID. Leave empty for cluster level rules.
        """
        return pulumi.get(self, "vm_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['FileArgs', 'File']

@pulumi.input_type
class FileArgs:
    def __init__(__self__, *,
                 content_type: pulumi.Input[str],
                 datastore_id: pulumi.Input[str],
                 node_name: pulumi.Input[str],
                 url: pulumi.Input[str],
                 checksum: Optional[pulumi.Input[str]] = None,
                 checksum_algorithm: Optional[pulumi.Input[str]] = None,
                 decompression_algorithm: Optional[pulumi.Input[str]] = None,
                 file_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 overwrite_unmanaged: Optional[pulumi.Input[bool]] = None,
                 upload_timeout: Optional[pulumi.Input[int]] = None,
                 verify: Optional[pulumi.Input[bool]] = None):
        """
        The set of arguments for constructing a File resource.
        :param pulumi.Input[str] content_type: The file content type. Must be `iso` for VM images or `vztmpl` for LXC images.
        :param pulumi.Input[str] datastore_id: The identifier for the target datastore.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[str] url: The URL to download the file from. Format `https?://.*`.
        :param pulumi.Input[str] checksum: The expected checksum of the file.
        :param pulumi.Input[str] checksum_algorithm: The algorithm to calculate the checksum of the file. Must be `md5` | `sha1` | `sha224` | `sha256` | `sha384` | `sha512`.
        :param pulumi.Input[str] decompression_algorithm: Decompress the downloaded file using the specified compression algorithm. Must be one of `gz` | `lzo` | `zst`.
        :param pulumi.Input[str] file_name: The file name. If not provided, it is calculated using `url`. PVE will raise 'wrong file extension' error for some popular extensions file `.raw` or `.qcow2`. Workaround is to use e.g. `.img` instead.
        :param pulumi.Input[bool] overwrite: If `true` and size of uploaded file is different, than size from `url` Content-Length header, file will be downloaded again. If `false`, there will be no checks.
        :param pulumi.Input[bool] overwrite_unmanaged: If `true` and a file with the same name already exists in the datastore, it will be deleted and the new file will be downloaded. If `false` and the file already exists, an error will be returned.
        :param pulumi.Input[int] upload_timeout: The file download timeout seconds. Default is 600 (10min).
        :param pulumi.Input[bool] verify: By default `true`. If `false`, no SSL/TLS certificates will be verified.
        """
        pulumi.set(__self__, "content_type", content_type)
        pulumi.set(__self__, "datastore_id", datastore_id)
        pulumi.set(__self__, "node_name", node_name)
        pulumi.set(__self__, "url", url)
        if checksum is not None:
            pulumi.set(__self__, "checksum", checksum)
        if checksum_algorithm is not None:
            pulumi.set(__self__, "checksum_algorithm", checksum_algorithm)
        if decompression_algorithm is not None:
            pulumi.set(__self__, "decompression_algorithm", decompression_algorithm)
        if file_name is not None:
            pulumi.set(__self__, "file_name", file_name)
        if overwrite is not None:
            pulumi.set(__self__, "overwrite", overwrite)
        if overwrite_unmanaged is not None:
            pulumi.set(__self__, "overwrite_unmanaged", overwrite_unmanaged)
        if upload_timeout is not None:
            pulumi.set(__self__, "upload_timeout", upload_timeout)
        if verify is not None:
            pulumi.set(__self__, "verify", verify)

    @property
    @pulumi.getter(name="contentType")
    def content_type(self) -> pulumi.Input[str]:
        """
        The file content type. Must be `iso` for VM images or `vztmpl` for LXC images.
        """
        return pulumi.get(self, "content_type")

    @content_type.setter
    def content_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "content_type", value)

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> pulumi.Input[str]:
        """
        The identifier for the target datastore.
        """
        return pulumi.get(self, "datastore_id")

    @datastore_id.setter
    def datastore_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "datastore_id", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Input[str]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter
    def url(self) -> pulumi.Input[str]:
        """
        The URL to download the file from. Format `https?://.*`.
        """
        return pulumi.get(self, "url")

    @url.setter
    def url(self, value: pulumi.Input[str]):
        pulumi.set(self, "url", value)

    @property
    @pulumi.getter
    def checksum(self) -> Optional[pulumi.Input[str]]:
        """
        The expected checksum of the file.
        """
        return pulumi.get(self, "checksum")

    @checksum.setter
    def checksum(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "checksum", value)

    @property
    @pulumi.getter(name="checksumAlgorithm")
    def checksum_algorithm(self) -> Optional[pulumi.Input[str]]:
        """
        The algorithm to calculate the checksum of the file. Must be `md5` | `sha1` | `sha224` | `sha256` | `sha384` | `sha512`.
        """
        return pulumi.get(self, "checksum_algorithm")

    @checksum_algorithm.setter
    def checksum_algorithm(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "checksum_algorithm", value)

    @property
    @pulumi.getter(name="decompressionAlgorithm")
    def decompression_algorithm(self) -> Optional[pulumi.Input[str]]:
        """
        Decompress the downloaded file using the specified compression algorithm. Must be one of `gz` | `lzo` | `zst`.
        """
        return pulumi.get(self, "decompression_algorithm")

    @decompression_algorithm.setter
    def decompression_algorithm(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "decompression_algorithm", value)

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> Optional[pulumi.Input[str]]:
        """
        The file name. If not provided, it is calculated using `url`. PVE will raise 'wrong file extension' error for some popular extensions file `.raw` or `.qcow2`. Workaround is to use e.g. `.img` instead.
        """
        return pulumi.get(self, "file_name")

    @file_name.setter
    def file_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_name", value)

    @property
    @pulumi.getter
    def overwrite(self) -> Optional[pulumi.Input[bool]]:
        """
        If `true` and size of uploaded file is different, than size from `url` Content-Length header, file will be downloaded again. If `false`, there will be no checks.
        """
        return pulumi.get(self, "overwrite")

    @overwrite.setter
    def overwrite(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite", value)

    @property
    @pulumi.getter(name="overwriteUnmanaged")
    def overwrite_unmanaged(self) -> Optional[pulumi.Input[bool]]:
        """
        If `true` and a file with the same name already exists in the datastore, it will be deleted and the new file will be downloaded. If `false` and the file already exists, an error will be returned.
        """
        return pulumi.get(self, "overwrite_unmanaged")

    @overwrite_unmanaged.setter
    def overwrite_unmanaged(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite_unmanaged", value)

    @property
    @pulumi.getter(name="uploadTimeout")
    def upload_timeout(self) -> Optional[pulumi.Input[int]]:
        """
        The file download timeout seconds. Default is 600 (10min).
        """
        return pulumi.get(self, "upload_timeout")

    @upload_timeout.setter
    def upload_timeout(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "upload_timeout", value)

    @property
    @pulumi.getter
    def verify(self) -> Optional[pulumi.Input[bool]]:
        """
        By default `true`. If `false`, no SSL/TLS certificates will be verified.
        """
        return pulumi.get(self, "verify")

    @verify.setter
    def verify(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "verify", value)


@pulumi.input_type
class _FileState:
    def __init__(__self__, *,
                 checksum: Optional[pulumi.Input[str]] = None,
                 checksum_algorithm: Optional[pulumi.Input[str]] = None,
                 content_type: Optional[pulumi.Input[str]] = None,
                 datastore_id: Optional[pulumi.Input[str]] = None,
                 decompression_algorithm: Optional[pulumi.Input[str]] = None,
                 file_name: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 overwrite_unmanaged: Optional[pulumi.Input[bool]] = None,
                 size: Optional[pulumi.Input[int]] = None,
                 upload_timeout: Optional[pulumi.Input[int]] = None,
                 url: Optional[pulumi.Input[str]] = None,
                 verify: Optional[pulumi.Input[bool]] = None):
        """
        Input properties used for looking up and filtering File resources.
        :param pulumi.Input[str] checksum: The expected checksum of the file.
        :param pulumi.Input[str] checksum_algorithm: The algorithm to calculate the checksum of the file. Must be `md5` | `sha1` | `sha224` | `sha256` | `sha384` | `sha512`.
        :param pulumi.Input[str] content_type: The file content type. Must be `iso` for VM images or `vztmpl` for LXC images.
        :param pulumi.Input[str] datastore_id: The identifier for the target datastore.
        :param pulumi.Input[str] decompression_algorithm: Decompress the downloaded file using the specified compression algorithm. Must be one of `gz` | `lzo` | `zst`.
        :param pulumi.Input[str] file_name: The file name. If not provided, it is calculated using `url`. PVE will raise 'wrong file extension' error for some popular extensions file `.raw` or `.qcow2`. Workaround is to use e.g. `.img` instead.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[bool] overwrite: If `true` and size of uploaded file is different, than size from `url` Content-Length header, file will be downloaded again. If `false`, there will be no checks.
        :param pulumi.Input[bool] overwrite_unmanaged: If `true` and a file with the same name already exists in the datastore, it will be deleted and the new file will be downloaded. If `false` and the file already exists, an error will be returned.
        :param pulumi.Input[int] size: The file size.
        :param pulumi.Input[int] upload_timeout: The file download timeout seconds. Default is 600 (10min).
        :param pulumi.Input[str] url: The URL to download the file from. Format `https?://.*`.
        :param pulumi.Input[bool] verify: By default `true`. If `false`, no SSL/TLS certificates will be verified.
        """
        if checksum is not None:
            pulumi.set(__self__, "checksum", checksum)
        if checksum_algorithm is not None:
            pulumi.set(__self__, "checksum_algorithm", checksum_algorithm)
        if content_type is not None:
            pulumi.set(__self__, "content_type", content_type)
        if datastore_id is not None:
            pulumi.set(__self__, "datastore_id", datastore_id)
        if decompression_algorithm is not None:
            pulumi.set(__self__, "decompression_algorithm", decompression_algorithm)
        if file_name is not None:
            pulumi.set(__self__, "file_name", file_name)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if overwrite is not None:
            pulumi.set(__self__, "overwrite", overwrite)
        if overwrite_unmanaged is not None:
            pulumi.set(__self__, "overwrite_unmanaged", overwrite_unmanaged)
        if size is not None:
            pulumi.set(__self__, "size", size)
        if upload_timeout is not None:
            pulumi.set(__self__, "upload_timeout", upload_timeout)
        if url is not None:
            pulumi.set(__self__, "url", url)
        if verify is not None:
            pulumi.set(__self__, "verify", verify)

    @property
    @pulumi.getter
    def checksum(self) -> Optional[pulumi.Input[str]]:
        """
        The expected checksum of the file.
        """
        return pulumi.get(self, "checksum")

    @checksum.setter
    def checksum(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "checksum", value)

    @property
    @pulumi.getter(name="checksumAlgorithm")
    def checksum_algorithm(self) -> Optional[pulumi.Input[str]]:
        """
        The algorithm to calculate the checksum of the file. Must be `md5` | `sha1` | `sha224` | `sha256` | `sha384` | `sha512`.
        """
        return pulumi.get(self, "checksum_algorithm")

    @checksum_algorithm.setter
    def checksum_algorithm(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "checksum_algorithm", value)

    @property
    @pulumi.getter(name="contentType")
    def content_type(self) -> Optional[pulumi.Input[str]]:
        """
        The file content type. Must be `iso` for VM images or `vztmpl` for LXC images.
        """
        return pulumi.get(self, "content_type")

    @content_type.setter
    def content_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "content_type", value)

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> Optional[pulumi.Input[str]]:
        """
        The identifier for the target datastore.
        """
        return pulumi.get(self, "datastore_id")

    @datastore_id.setter
    def datastore_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "datastore_id", value)

    @property
    @pulumi.getter(name="decompressionAlgorithm")
    def decompression_algorithm(self) -> Optional[pulumi.Input[str]]:
        """
        Decompress the downloaded file using the specified compression algorithm. Must be one of `gz` | `lzo` | `zst`.
        """
        return pulumi.get(self, "decompression_algorithm")

    @decompression_algorithm.setter
    def decompression_algorithm(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "decompression_algorithm", value)

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> Optional[pulumi.Input[str]]:
        """
        The file name. If not provided, it is calculated using `url`. PVE will raise 'wrong file extension' error for some popular extensions file `.raw` or `.qcow2`. Workaround is to use e.g. `.img` instead.
        """
        return pulumi.get(self, "file_name")

    @file_name.setter
    def file_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_name", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter
    def overwrite(self) -> Optional[pulumi.Input[bool]]:
        """
        If `true` and size of uploaded file is different, than size from `url` Content-Length header, file will be downloaded again. If `false`, there will be no checks.
        """
        return pulumi.get(self, "overwrite")

    @overwrite.setter
    def overwrite(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite", value)

    @property
    @pulumi.getter(name="overwriteUnmanaged")
    def overwrite_unmanaged(self) -> Optional[pulumi.Input[bool]]:
        """
        If `true` and a file with the same name already exists in the datastore, it will be deleted and the new file will be downloaded. If `false` and the file already exists, an error will be returned.
        """
        return pulumi.get(self, "overwrite_unmanaged")

    @overwrite_unmanaged.setter
    def overwrite_unmanaged(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite_unmanaged", value)

    @property
    @pulumi.getter
    def size(self) -> Optional[pulumi.Input[int]]:
        """
        The file size.
        """
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "size", value)

    @property
    @pulumi.getter(name="uploadTimeout")
    def upload_timeout(self) -> Optional[pulumi.Input[int]]:
        """
        The file download timeout seconds. Default is 600 (10min).
        """
        return pulumi.get(self, "upload_timeout")

    @upload_timeout.setter
    def upload_timeout(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "upload_timeout", value)

    @property
    @pulumi.getter
    def url(self) -> Optional[pulumi.Input[str]]:
        """
        The URL to download the file from. Format `https?://.*`.
        """
        return pulumi.get(self, "url")

    @url.setter
    def url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "url", value)

    @property
    @pulumi.getter
    def verify(self) -> Optional[pulumi.Input[bool]]:
        """
        By default `true`. If `false`, no SSL/TLS certificates will be verified.
        """
        return pulumi.get(self, "verify")

    @verify.setter
    def verify(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "verify", value)


class File(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 checksum: Optional[pulumi.Input[str]] = None,
                 checksum_algorithm: Optional[pulumi.Input[str]] = None,
                 content_type: Optional[pulumi.Input[str]] = None,
                 datastore_id: Optional[pulumi.Input[str]] = None,
                 decompression_algorithm: Optional[pulumi.Input[str]] = None,
                 file_name: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 overwrite_unmanaged: Optional[pulumi.Input[bool]] = None,
                 upload_timeout: Optional[pulumi.Input[int]] = None,
                 url: Optional[pulumi.Input[str]] = None,
                 verify: Optional[pulumi.Input[bool]] = None,
                 __props__=None):
        """
        Manages files upload using PVE download-url API. It can be fully compatible and faster replacement for image files created using `Storage.File`. Supports images for VMs (ISO images) and LXC (CT Templates).

        ## Example Usage

        <!--Start PulumiCodeChooser -->
        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        release20231228_debian12_bookworm_qcow2_img = proxmoxve.download.File("release20231228Debian12BookwormQcow2Img",
            checksum="d2fbcf11fb28795842e91364d8c7b69f1870db09ff299eb94e4fbbfa510eb78d141e74c1f4bf6dfa0b7e33d0c3b66e6751886feadb4e9916f778bab1776bdf1b",
            checksum_algorithm="sha512",
            content_type="iso",
            datastore_id="local",
            file_name="debian-12-generic-amd64-20231228-1609.img",
            node_name="pve",
            url="https://cloud.debian.org/images/cloud/bookworm/20231228-1609/debian-12-generic-amd64-20231228-1609.qcow2")
        latest_debian12_bookworm_qcow2_img = proxmoxve.download.File("latestDebian12BookwormQcow2Img",
            content_type="iso",
            datastore_id="local",
            file_name="debian-12-generic-amd64.qcow2.img",
            node_name="pve",
            url="https://cloud.debian.org/images/cloud/bookworm/latest/debian-12-generic-amd64.qcow2")
        latest_ubuntu22_jammy_qcow2_img = proxmoxve.download.File("latestUbuntu22JammyQcow2Img",
            content_type="iso",
            datastore_id="local",
            node_name="pve",
            url="https://cloud-images.ubuntu.com/jammy/current/jammy-server-cloudimg-amd64.img")
        latest_static_ubuntu24_noble_qcow2_img = proxmoxve.download.File("latestStaticUbuntu24NobleQcow2Img",
            content_type="iso",
            datastore_id="local",
            node_name="pve",
            overwrite=False,
            url="https://cloud-images.ubuntu.com/noble/current/noble-server-cloudimg-amd64.img")
        release20231211_ubuntu22_jammy_lxc_img = proxmoxve.download.File("release20231211Ubuntu22JammyLxcImg",
            checksum="c9997dcfea5d826fd04871f960c513665f2e87dd7450bba99f68a97e60e4586e",
            checksum_algorithm="sha256",
            content_type="vztmpl",
            datastore_id="local",
            node_name="pve",
            upload_timeout=4444,
            url="https://cloud-images.ubuntu.com/releases/22.04/release-20231211/ubuntu-22.04-server-cloudimg-amd64-root.tar.xz")
        latest_ubuntu22_jammy_lxc_img = proxmoxve.download.File("latestUbuntu22JammyLxcImg",
            content_type="vztmpl",
            datastore_id="local",
            node_name="pve",
            url="https://cloud-images.ubuntu.com/jammy/current/jammy-server-cloudimg-amd64.tar.gz")
        ```
        <!--End PulumiCodeChooser -->

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] checksum: The expected checksum of the file.
        :param pulumi.Input[str] checksum_algorithm: The algorithm to calculate the checksum of the file. Must be `md5` | `sha1` | `sha224` | `sha256` | `sha384` | `sha512`.
        :param pulumi.Input[str] content_type: The file content type. Must be `iso` for VM images or `vztmpl` for LXC images.
        :param pulumi.Input[str] datastore_id: The identifier for the target datastore.
        :param pulumi.Input[str] decompression_algorithm: Decompress the downloaded file using the specified compression algorithm. Must be one of `gz` | `lzo` | `zst`.
        :param pulumi.Input[str] file_name: The file name. If not provided, it is calculated using `url`. PVE will raise 'wrong file extension' error for some popular extensions file `.raw` or `.qcow2`. Workaround is to use e.g. `.img` instead.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[bool] overwrite: If `true` and size of uploaded file is different, than size from `url` Content-Length header, file will be downloaded again. If `false`, there will be no checks.
        :param pulumi.Input[bool] overwrite_unmanaged: If `true` and a file with the same name already exists in the datastore, it will be deleted and the new file will be downloaded. If `false` and the file already exists, an error will be returned.
        :param pulumi.Input[int] upload_timeout: The file download timeout seconds. Default is 600 (10min).
        :param pulumi.Input[str] url: The URL to download the file from. Format `https?://.*`.
        :param pulumi.Input[bool] verify: By default `true`. If `false`, no SSL/TLS certificates will be verified.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: FileArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages files upload using PVE download-url API. It can be fully compatible and faster replacement for image files created using `Storage.File`. Supports images for VMs (ISO images) and LXC (CT Templates).

        ## Example Usage

        <!--Start PulumiCodeChooser -->
        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        release20231228_debian12_bookworm_qcow2_img = proxmoxve.download.File("release20231228Debian12BookwormQcow2Img",
            checksum="d2fbcf11fb28795842e91364d8c7b69f1870db09ff299eb94e4fbbfa510eb78d141e74c1f4bf6dfa0b7e33d0c3b66e6751886feadb4e9916f778bab1776bdf1b",
            checksum_algorithm="sha512",
            content_type="iso",
            datastore_id="local",
            file_name="debian-12-generic-amd64-20231228-1609.img",
            node_name="pve",
            url="https://cloud.debian.org/images/cloud/bookworm/20231228-1609/debian-12-generic-amd64-20231228-1609.qcow2")
        latest_debian12_bookworm_qcow2_img = proxmoxve.download.File("latestDebian12BookwormQcow2Img",
            content_type="iso",
            datastore_id="local",
            file_name="debian-12-generic-amd64.qcow2.img",
            node_name="pve",
            url="https://cloud.debian.org/images/cloud/bookworm/latest/debian-12-generic-amd64.qcow2")
        latest_ubuntu22_jammy_qcow2_img = proxmoxve.download.File("latestUbuntu22JammyQcow2Img",
            content_type="iso",
            datastore_id="local",
            node_name="pve",
            url="https://cloud-images.ubuntu.com/jammy/current/jammy-server-cloudimg-amd64.img")
        latest_static_ubuntu24_noble_qcow2_img = proxmoxve.download.File("latestStaticUbuntu24NobleQcow2Img",
            content_type="iso",
            datastore_id="local",
            node_name="pve",
            overwrite=False,
            url="https://cloud-images.ubuntu.com/noble/current/noble-server-cloudimg-amd64.img")
        release20231211_ubuntu22_jammy_lxc_img = proxmoxve.download.File("release20231211Ubuntu22JammyLxcImg",
            checksum="c9997dcfea5d826fd04871f960c513665f2e87dd7450bba99f68a97e60e4586e",
            checksum_algorithm="sha256",
            content_type="vztmpl",
            datastore_id="local",
            node_name="pve",
            upload_timeout=4444,
            url="https://cloud-images.ubuntu.com/releases/22.04/release-20231211/ubuntu-22.04-server-cloudimg-amd64-root.tar.xz")
        latest_ubuntu22_jammy_lxc_img = proxmoxve.download.File("latestUbuntu22JammyLxcImg",
            content_type="vztmpl",
            datastore_id="local",
            node_name="pve",
            url="https://cloud-images.ubuntu.com/jammy/current/jammy-server-cloudimg-amd64.tar.gz")
        ```
        <!--End PulumiCodeChooser -->

        :param str resource_name: The name of the resource.
        :param FileArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FileArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 checksum: Optional[pulumi.Input[str]] = None,
                 checksum_algorithm: Optional[pulumi.Input[str]] = None,
                 content_type: Optional[pulumi.Input[str]] = None,
                 datastore_id: Optional[pulumi.Input[str]] = None,
                 decompression_algorithm: Optional[pulumi.Input[str]] = None,
                 file_name: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 overwrite_unmanaged: Optional[pulumi.Input[bool]] = None,
                 upload_timeout: Optional[pulumi.Input[int]] = None,
                 url: Optional[pulumi.Input[str]] = None,
                 verify: Optional[pulumi.Input[bool]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FileArgs.__new__(FileArgs)

            __props__.__dict__["checksum"] = checksum
            __props__.__dict__["checksum_algorithm"] = checksum_algorithm
            if content_type is None and not opts.urn:
                raise TypeError("Missing required property 'content_type'")
            __props__.__dict__["content_type"] = content_type
            if datastore_id is None and not opts.urn:
                raise TypeError("Missing required property 'datastore_id'")
            __props__.__dict__["datastore_id"] = datastore_id
            __props__.__dict__["decompression_algorithm"] = decompression_algorithm
            __props__.__dict__["file_name"] = file_name
            if node_name is None and not opts.urn:
                raise TypeError("Missing required property 'node_name'")
            __props__.__dict__["node_name"] = node_name
            __props__.__dict__["overwrite"] = overwrite
            __props__.__dict__["overwrite_unmanaged"] = overwrite_unmanaged
            __props__.__dict__["upload_timeout"] = upload_timeout
            if url is None and not opts.urn:
                raise TypeError("Missing required property 'url'")
            __props__.__dict__["url"] = url
            __props__.__dict__["verify"] = verify
            __props__.__dict__["size"] = None
        super(File, __self__).__init__(
            'proxmoxve:Download/file:File',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            checksum: Optional[pulumi.Input[str]] = None,
            checksum_algorithm: Optional[pulumi.Input[str]] = None,
            content_type: Optional[pulumi.Input[str]] = None,
            datastore_id: Optional[pulumi.Input[str]] = None,
            decompression_algorithm: Optional[pulumi.Input[str]] = None,
            file_name: Optional[pulumi.Input[str]] = None,
            node_name: Optional[pulumi.Input[str]] = None,
            overwrite: Optional[pulumi.Input[bool]] = None,
            overwrite_unmanaged: Optional[pulumi.Input[bool]] = None,
            size: Optional[pulumi.Input[int]] = None,
            upload_timeout: Optional[pulumi.Input[int]] = None,
            url: Optional[pulumi.Input[str]] = None,
            verify: Optional[pulumi.Input[bool]] = None) -> 'File':
        """
        Get an existing File resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] checksum: The expected checksum of the file.
        :param pulumi.Input[str] checksum_algorithm: The algorithm to calculate the checksum of the file. Must be `md5` | `sha1` | `sha224` | `sha256` | `sha384` | `sha512`.
        :param pulumi.Input[str] content_type: The file content type. Must be `iso` for VM images or `vztmpl` for LXC images.
        :param pulumi.Input[str] datastore_id: The identifier for the target datastore.
        :param pulumi.Input[str] decompression_algorithm: Decompress the downloaded file using the specified compression algorithm. Must be one of `gz` | `lzo` | `zst`.
        :param pulumi.Input[str] file_name: The file name. If not provided, it is calculated using `url`. PVE will raise 'wrong file extension' error for some popular extensions file `.raw` or `.qcow2`. Workaround is to use e.g. `.img` instead.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[bool] overwrite: If `true` and size of uploaded file is different, than size from `url` Content-Length header, file will be downloaded again. If `false`, there will be no checks.
        :param pulumi.Input[bool] overwrite_unmanaged: If `true` and a file with the same name already exists in the datastore, it will be deleted and the new file will be downloaded. If `false` and the file already exists, an error will be returned.
        :param pulumi.Input[int] size: The file size.
        :param pulumi.Input[int] upload_timeout: The file download timeout seconds. Default is 600 (10min).
        :param pulumi.Input[str] url: The URL to download the file from. Format `https?://.*`.
        :param pulumi.Input[bool] verify: By default `true`. If `false`, no SSL/TLS certificates will be verified.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FileState.__new__(_FileState)

        __props__.__dict__["checksum"] = checksum
        __props__.__dict__["checksum_algorithm"] = checksum_algorithm
        __props__.__dict__["content_type"] = content_type
        __props__.__dict__["datastore_id"] = datastore_id
        __props__.__dict__["decompression_algorithm"] = decompression_algorithm
        __props__.__dict__["file_name"] = file_name
        __props__.__dict__["node_name"] = node_name
        __props__.__dict__["overwrite"] = overwrite
        __props__.__dict__["overwrite_unmanaged"] = overwrite_unmanaged
        __props__.__dict__["size"] = size
        __props__.__dict__["upload_timeout"] = upload_timeout
        __props__.__dict__["url"] = url
        __props__.__dict__["verify"] = verify
        return File(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def checksum(self) -> pulumi.Output[Optional[str]]:
        """
        The expected checksum of the file.
        """
        return pulumi.get(self, "checksum")

    @property
    @pulumi.getter(name="checksumAlgorithm")
    def checksum_algorithm(self) -> pulumi.Output[Optional[str]]:
        """
        The algorithm to calculate the checksum of the file. Must be `md5` | `sha1` | `sha224` | `sha256` | `sha384` | `sha512`.
        """
        return pulumi.get(self, "checksum_algorithm")

    @property
    @pulumi.getter(name="contentType")
    def content_type(self) -> pulumi.Output[str]:
        """
        The file content type. Must be `iso` for VM images or `vztmpl` for LXC images.
        """
        return pulumi.get(self, "content_type")

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> pulumi.Output[str]:
        """
        The identifier for the target datastore.
        """
        return pulumi.get(self, "datastore_id")

    @property
    @pulumi.getter(name="decompressionAlgorithm")
    def decompression_algorithm(self) -> pulumi.Output[Optional[str]]:
        """
        Decompress the downloaded file using the specified compression algorithm. Must be one of `gz` | `lzo` | `zst`.
        """
        return pulumi.get(self, "decompression_algorithm")

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> pulumi.Output[str]:
        """
        The file name. If not provided, it is calculated using `url`. PVE will raise 'wrong file extension' error for some popular extensions file `.raw` or `.qcow2`. Workaround is to use e.g. `.img` instead.
        """
        return pulumi.get(self, "file_name")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Output[str]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def overwrite(self) -> pulumi.Output[bool]:
        """
        If `true` and size of uploaded file is different, than size from `url` Content-Length header, file will be downloaded again. If `false`, there will be no checks.
        """
        return pulumi.get(self, "overwrite")

    @property
    @pulumi.getter(name="overwriteUnmanaged")
    def overwrite_unmanaged(self) -> pulumi.Output[bool]:
        """
        If `true` and a file with the same name already exists in the datastore, it will be deleted and the new file will be downloaded. If `false` and the file already exists, an error will be returned.
        """
        return pulumi.get(self, "overwrite_unmanaged")

    @property
    @pulumi.getter
    def size(self) -> pulumi.Output[int]:
        """
        The file size.
        """
        return pulumi.get(self, "size")

    @property
    @pulumi.getter(name="uploadTimeout")
    def upload_timeout(self) -> pulumi.Output[int]:
        """
        The file download timeout seconds. Default is 600 (10min).
        """
        return pulumi.get(self, "upload_timeout")

    @property
    @pulumi.getter
    def url(self) -> pulumi.Output[str]:
        """
        The URL to download the file from. Format `https?://.*`.
        """
        return pulumi.get(self, "url")

    @property
    @pulumi.getter
    def verify(self) -> pulumi.Output[bool]:
        """
        By default `true`. If `false`, no SSL/TLS certificates will be verified.
        """
        return pulumi.get(self, "verify")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'FileSourceFile',
    'FileSourceRaw',
]

@pulumi.output_type
class FileSourceFile(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "fileName":
            suggest = "file_name"
        elif key == "minTls":
            suggest = "min_tls"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in FileSourceFile. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        FileSourceFile.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        FileSourceFile.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 path: str,
                 changed: Optional[bool] = None,
                 checksum: Optional[str] = None,
                 file_name: Optional[str] = None,
                 insecure: Optional[bool] = None,
                 min_tls: Optional[str] = None):
        """
        :param str path: A path to a local file or a URL.
        :param bool changed: Whether the source file has changed since the last run
        :param str checksum: The SHA256 checksum of the source file.
        :param str file_name: The file name.
        :param bool insecure: Whether to skip the TLS verification step for
               HTTPS sources (defaults to `false`).
        :param str min_tls: The minimum required TLS version for HTTPS
               sources. "Supported values: `1.0|1.1|1.2|1.3` (defaults to `1.3`).
        """
        pulumi.set(__self__, "path", path)
        if changed is not None:
            pulumi.set(__self__, "changed", changed)
        if checksum is not None:
            pulumi.set(__self__, "checksum", checksum)
        if file_name is not None:
            pulumi.set(__self__, "file_name", file_name)
        if insecure is not None:
            pulumi.set(__self__, "insecure", insecure)
        if min_tls is not None:
            pulumi.set(__self__, "min_tls", min_tls)

    @property
    @pulumi.getter
    def path(self) -> str:
        """
        A path to a local file or a URL.
        """
        return pulumi.get(self, "path")

    @property
    @pulumi.getter
    def changed(self) -> Optional[bool]:
        """
        Whether the source file has changed since the last run
        """
        return pulumi.get(self, "changed")

    @property
    @pulumi.getter
    def checksum(self) -> Optional[str]:
        """
        The SHA256 checksum of the source file.
        """
        return pulumi.get(self, "checksum")

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> Optional[str]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @property
    @pulumi.getter
    def insecure(self) -> Optional[bool]:
        """
        Whether to skip the TLS verification step for
        HTTPS sources (defaults to `false`).
        """
        return pulumi.get(self, "insecure")

    @property
    @pulumi.getter(name="minTls")
    def min_tls(self) -> Optional[str]:
        """
        The minimum required TLS version for HTTPS
        sources. "Supported values: `1.0|1.1|1.2|1.3` (defaults to `1.3`).
        """
        return pulumi.get(self, "min_tls")


@pulumi.output_type
class FileSourceRaw(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "fileName":
            suggest = "file_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in FileSourceRaw. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        FileSourceRaw.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        FileSourceRaw.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 data: str,
                 file_name: str,
                 resize: Optional[int] = None):
        """
        :param str data: The raw data.
        :param str file_name: The file name.
        :param int resize: The number of bytes to resize the file to.
        """
        pulumi.set(__self__, "data", data)
        pulumi.set(__self__, "file_name", file_name)
        if resize is not None:
            pulumi.set(__self__, "resize", resize)

    @property
    @pulumi.getter
    def data(self) -> str:
        """
        The raw data.
        """
        return pulumi.get(self, "data")

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> str:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @property
    @pulumi.getter
    def resize(self) -> Optional[int]:
        """
        The number of bytes to resize the file to.
        """
        return pulumi.get(self, "resize")





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetUserResult',
    'AwaitableGetUserResult',
    'get_user',
    'get_user_output',
]

@pulumi.output_type
class GetUserResult:
    """
    A collection of values returned by getUser.
    """
    def __init__(__self__, acls=None, comment=None, email=None, enabled=None, expiration_date=None, first_name=None, groups=None, id=None, keys=None, last_name=None, user_id=None):
        if acls and not isinstance(acls, list):
            raise TypeError("Expected argument 'acls' to be a list")
        pulumi.set(__self__, "acls", acls)
        if comment and not isinstance(comment, str):
            raise TypeError("Expected argument 'comment' to be a str")
        pulumi.set(__self__, "comment", comment)
        if email and not isinstance(email, str):
            raise TypeError("Expected argument 'email' to be a str")
        pulumi.set(__self__, "email", email)
        if enabled and not isinstance(enabled, bool):
            raise TypeError("Expected argument 'enabled' to be a bool")
        pulumi.set(__self__, "enabled", enabled)
        if expiration_date and not isinstance(expiration_date, str):
            raise TypeError("Expected argument 'expiration_date' to be a str")
        pulumi.set(__self__, "expiration_date", expiration_date)
        if first_name and not isinstance(first_name, str):
            raise TypeError("Expected argument 'first_name' to be a str")
        pulumi.set(__self__, "first_name", first_name)
        if groups and not isinstance(groups, list):
            raise TypeError("Expected argument 'groups' to be a list")
        pulumi.set(__self__, "groups", groups)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if keys and not isinstance(keys, str):
            raise TypeError("Expected argument 'keys' to be a str")
        pulumi.set(__self__, "keys", keys)
        if last_name and not isinstance(last_name, str):
            raise TypeError("Expected argument 'last_name' to be a str")
        pulumi.set(__self__, "last_name", last_name)
        if user_id and not isinstance(user_id, str):
            raise TypeError("Expected argument 'user_id' to be a str")
        pulumi.set(__self__, "user_id", user_id)

    @property
    @pulumi.getter
    def acls(self) -> Sequence['outputs.GetUserAclResult']:
        """
        The access control list.
        """
        return pulumi.get(self, "acls")

    @property
    @pulumi.getter
    def comment(self) -> str:
        """
        The user comment.
        """
        return pulumi.get(self, "comment")

    @property
    @pulumi.getter
    def email(self) -> str:
        """
        The user's email address.
        """
        return pulumi.get(self, "email")

    @property
    @pulumi.getter
    def enabled(self) -> bool:
        """
        Whether the user account is enabled.
        """
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> str:
        """
        The user account's expiration date (RFC 3339).
        """
        return pulumi.get(self, "expiration_date")

    @property
    @pulumi.getter(name="firstName")
    def first_name(self) -> str:
        """
        The user's first name.
        """
        return pulumi.get(self, "first_name")

    @property
    @pulumi.getter
    def groups(self) -> Sequence[str]:
        """
        The user's groups.
        """
        return pulumi.get(self, "groups")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def keys(self) -> str:
        """
        The user's keys.
        """
        return pulumi.get(self, "keys")

    @property
    @pulumi.getter(name="lastName")
    def last_name(self) -> str:
        """
        The user's last name.
        """
        return pulumi.get(self, "last_name")

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> str:
        return pulumi.get(self, "user_id")


class AwaitableGetUserResult(GetUserResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetUserResult(
            acls=self.acls,
            comment=self.comment,
            email=self.email,
            enabled=self.enabled,
            expiration_date=self.expiration_date,
            first_name=self.first_name,
            groups=self.groups,
            id=self.id,
            keys=self.keys,
            last_name=self.last_name,
            user_id=self.user_id)


def get_user(user_id: Optional[str] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetUserResult:
    """
    Retrieves information about a specific user.

    ## Example Usage

    <!--Start PulumiCodeChooser -->
    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    operations_user = proxmoxve.Permission.get_user(user_id="operation@pam")
    ```
    <!--End PulumiCodeChooser -->


    :param str user_id: The user identifier.
    """
    __args__ = dict()
    __args__['userId'] = user_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:Permission/getUser:getUser', __args__, opts=opts, typ=GetUserResult).value

    return AwaitableGetUserResult(
        acls=pulumi.get(__ret__, 'acls'),
        comment=pulumi.get(__ret__, 'comment'),
        email=pulumi.get(__ret__, 'email'),
        enabled=pulumi.get(__ret__, 'enabled'),
        expiration_date=pulumi.get(__ret__, 'expiration_date'),
        first_name=pulumi.get(__ret__, 'first_name'),
        groups=pulumi.get(__ret__, 'groups'),
        id=pulumi.get(__ret__, 'id'),
        keys=pulumi.get(__ret__, 'keys'),
        last_name=pulumi.get(__ret__, 'last_name'),
        user_id=pulumi.get(__ret__, 'user_id'))


@_utilities.lift_output_func(get_user)
def get_user_output(user_id: Optional[pulumi.Input[str]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetUserResult]:
    """
    Retrieves information about a specific user.

    ## Example Usage

    <!--Start PulumiCodeChooser -->
    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    operations_user = proxmoxve.Permission.get_user(user_id="operation@pam")
    ```
    <!--End PulumiCodeChooser -->


    :param str user_id: The user identifier.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetNodesResult',
    'AwaitableGetNodesResult',
    'get_nodes',
    'get_nodes_output',
]

@pulumi.output_type
class GetNodesResult:
    """
    A collection of values returned by getNodes.
    """
    def __init__(__self__, cpu_counts=None, cpu_utilizations=None, id=None, memory_availables=None, memory_useds=None, names=None, onlines=None, ssl_fingerprints=None, support_levels=None, uptimes=None):
        if cpu_counts and not isinstance(cpu_counts, list):
            raise TypeError("Expected argument 'cpu_counts' to be a list")
        pulumi.set(__self__, "cpu_counts", cpu_counts)
        if cpu_utilizations and not isinstance(cpu_utilizations, list):
            raise TypeError("Expected argument 'cpu_utilizations' to be a list")
        pulumi.set(__self__, "cpu_utilizations", cpu_utilizations)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if memory_availables and not isinstance(memory_availables, list):
            raise TypeError("Expected argument 'memory_availables' to be a list")
        pulumi.set(__self__, "memory_availables", memory_availables)
        if memory_useds and not isinstance(memory_useds, list):
            raise TypeError("Expected argument 'memory_useds' to be a list")
        pulumi.set(__self__, "memory_useds", memory_useds)
        if names and not isinstance(names, list):
            raise TypeError("Expected argument 'names' to be a list")
        pulumi.set(__self__, "names", names)
        if onlines and not isinstance(onlines, list):
            raise TypeError("Expected argument 'onlines' to be a list")
        pulumi.set(__self__, "onlines", onlines)
        if ssl_fingerprints and not isinstance(ssl_fingerprints, list):
            raise TypeError("Expected argument 'ssl_fingerprints' to be a list")
        pulumi.set(__self__, "ssl_fingerprints", ssl_fingerprints)
        if support_levels and not isinstance(support_levels, list):
            raise TypeError("Expected argument 'support_levels' to be a list")
        pulumi.set(__self__, "support_levels", support_levels)
        if uptimes and not isinstance(uptimes, list):
            raise TypeError("Expected argument 'uptimes' to be a list")
        pulumi.set(__self__, "uptimes", uptimes)

    @property
    @pulumi.getter(name="cpuCounts")
    def cpu_counts(self) -> Sequence[int]:
        """
        The CPU count for each node.
        """
        return pulumi.get(self, "cpu_counts")

    @property
    @pulumi.getter(name="cpuUtilizations")
    def cpu_utilizations(self) -> Sequence[float]:
        """
        The CPU utilization on each node.
        """
        return pulumi.get(self, "cpu_utilizations")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="memoryAvailables")
    def memory_availables(self) -> Sequence[int]:
        """
        The memory available on each node.
        """
        return pulumi.get(self, "memory_availables")

    @property
    @pulumi.getter(name="memoryUseds")
    def memory_useds(self) -> Sequence[int]:
        """
        The memory used on each node.
        """
        return pulumi.get(self, "memory_useds")

    @property
    @pulumi.getter
    def names(self) -> Sequence[str]:
        """
        The node names.
        """
        return pulumi.get(self, "names")

    @property
    @pulumi.getter
    def onlines(self) -> Sequence[bool]:
        """
        Whether a node is online.
        """
        return pulumi.get(self, "onlines")

    @property
    @pulumi.getter(name="sslFingerprints")
    def ssl_fingerprints(self) -> Sequence[str]:
        """
        The SSL fingerprint for each node.
        """
        return pulumi.get(self, "ssl_fingerprints")

    @property
    @pulumi.getter(name="supportLevels")
    def support_levels(self) -> Sequence[str]:
        """
        The support level for each node.
        """
        return pulumi.get(self, "support_levels")

    @property
    @pulumi.getter
    def uptimes(self) -> Sequence[int]:
        """
        The uptime in seconds for each node.
        """
        return pulumi.get(self, "uptimes")


class AwaitableGetNodesResult(GetNodesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNodesResult(
            cpu_counts=self.cpu_counts,
            cpu_utilizations=self.cpu_utilizations,
            id=self.id,
            memory_availables=self.memory_availables,
            memory_useds=self.memory_useds,
            names=self.names,
            onlines=self.onlines,
            ssl_fingerprints=self.ssl_fingerprints,
            support_levels=self.support_levels,
            uptimes=self.uptimes)


def get_nodes(opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetNodesResult:
    """
    Retrieves information about all available nodes.

    ## Example Usage

    <!--Start PulumiCodeChooser -->
    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    available_nodes = proxmoxve.Cluster.get_nodes()
    ```
    <!--End PulumiCodeChooser -->
    """
    __args__ = dict()
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:Cluster/getNodes:getNodes', __args__, opts=opts, typ=GetNodesResult).value

    return AwaitableGetNodesResult(
        cpu_counts=pulumi.get(__ret__, 'cpu_counts'),
        cpu_utilizations=pulumi.get(__ret__, 'cpu_utilizations'),
        id=pulumi.get(__ret__, 'id'),
        memory_availables=pulumi.get(__ret__, 'memory_availables'),
        memory_useds=pulumi.get(__ret__, 'memory_useds'),
        names=pulumi.get(__ret__, 'names'),
        onlines=pulumi.get(__ret__, 'onlines'),
        ssl_fingerprints=pulumi.get(__ret__, 'ssl_fingerprints'),
        support_levels=pulumi.get(__ret__, 'support_levels'),
        uptimes=pulumi.get(__ret__, 'uptimes'))


@_utilities.lift_output_func(get_nodes)
def get_nodes_output(opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetNodesResult]:
    """
    Retrieves information about all available nodes.

    ## Example Usage

    <!--Start PulumiCodeChooser -->
    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    available_nodes = proxmoxve.Cluster.get_nodes()
    ```
    <!--End PulumiCodeChooser -->
    """
    ...
